#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

### Big 'ol hack to make an anaconda-like screen.  eventually, I'd
### expect that all this goes into Anaconda itself.  Or
### something. Also, the packing is all very odd to match anaconda's
### packing. -jrb

import sys

if __name__ == '__main__':
    sys.path = sys.path + ['..', '../lib', '../replication-client', '../snapshooter']

import gtk
import os
import config
import MACAddress
import gtk.glade
import LDAPUtils

# We need to make sure that we modify the ROOTPATH before importing
# replicator
config.ROOTPATH = '/mnt/sysimage'
import replicator

# Borrow some code from Anaconda
sys.path.append ('/usr/lib/anaconda/')
from gui import TextViewBrowser
from gui import WaitWindow
import htmlbuffer

def get_help_text ():
    text_view = TextViewBrowser()
    buffer = htmlbuffer.HTMLBuffer()
    text_view.set_buffer (buffer.get_buffer())
    buffer.feed(
"""<h2>Preparing Client</h2><p>Preparing this computer to be a
Stateless Managed Linux client.  This will copy the latest version of the configuration
from the server and set up GRUB to use this copy.</p> """)
    return text_view


def get_pixmap_path (filename):
    path = '/usr/share/anaconda/pixmaps/' + filename
    if os.path.isfile (path):
        return path
    raise IOError

def remove_dialog (r, update, dialog):
    dialog.pop ()

def update_cb (r, update, gui):
    if gui.firsttime:
        gui.label.set_text ('Copying version %s of %s from the server to disk'%(gui.provisioning.snapshot_name, gui.provisioning.config_name))
        gui.firsttime = False
    gui.progress.set_percentage (update)

def setup_initial_disks ():
    # unmount the remote drives.s
    replicator.umount ("/mnt/sysimage/reserve-root")
    replicator.umount ("/mnt/sysimage/reserve-boot")


    # Relabel the drives
    reserve_root_device = replicator.get_device_by_label ('/reserve-root')
    assert (reserve_root_device)
    replicator.relabel_device (reserve_root_device, "RESERVE_ROOT")
    reserve_boot_device = replicator.get_device_by_label ('/reserve-boot')
    assert (reserve_boot_device)
    replicator.relabel_device (reserve_boot_device, "RESERVE_BOOT")

def get_glade_path(filename):
    return "./%s"%filename

def on_replication_finished(replicator, success):
    if success and not replicator.debug:
        gtk.mainquit()    

class BootstrapGUI:
    def __init__(self, ldap_server, root_dn, debug=False):
        self.firsttime = False
        self.ldap_server = ldap_server
        self.ldap_uri = "ldap://%s"%ldap_server
        self.root_dn = root_dn
        self.debug = debug

    def get_rsync_server(self, replicator):
        config_name   = replicator.provisioning.config_name
        snapshot_name = replicator.provisioning.snapshot_name
        return replicator.stateless_service.get_snapshot_location(config_name, snapshot_name, "rsync")

    def run(self):
        self.create_window(self.debug)

        dialog = WaitWindow ("Obtaining provisioning information from %s" % self.ldap_server, "Obtaining provisioning information for %s from %s."%(MACAddress.get_eth_mac_address(), self.ldap_server))

        if not self.debug:
            setup_initial_disks ()

        # FIXME: supposed to pass a StatelessConfig here
        r = replicator.BootstrapReplicator (self.ldap_uri, self.root_dn, self.debug)

        dialog.pop ()

        self.provisioning = r.provisioning

        #self.server_text.set_text(self.get_rsync_server(r))
        if r.provisioning.config_name:
            self.configuration_id_text.set_text(r.provisioning.config_name)
        else:
            self.configuration_id_text.set_text("(unknown)")
        if r.provisioning.snapshot_name:
            self.version_text.set_text(r.provisioning.snapshot_name)
        else:
            self.version_text.set_text("(unknown)")
            
        if r.provisioning.is_valid():
            dialog = WaitWindow ("Preparing file list for %s" % r.provisioning.config_name, "Preparing the list of files to be transfered from %s.  This may take a few minutes to complete."%self.get_rsync_server(r))

            r.connect ('update', update_cb, self)
            r.connect ('update', remove_dialog, dialog)
            r.connect ('state-change', self.state_change_cb)
            r.connect ('finished', on_replication_finished)

            r.begin_update_client ()
        else:
            self.label.set_text ("")

        gtk.mainloop ()

    def create_data_table(self):
        data_table = gtk.Table (4, 2, gtk.FALSE)
        data_table.set_row_spacings(6)
        data_table.set_col_spacings(6)

        client_label = gtk.Label('')
        client_label.set_alignment (1.0, 0.5)
        client_label.set_markup('<b>Client</b>')
        data_table.attach (client_label, 0, 1, 0, 1,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        self.client_text = gtk.Label(MACAddress.get_eth_mac_address())
        self.client_text.set_alignment (1.0, 0.5)
        data_table.attach (self.client_text, 1, 2, 0, 1,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        server_label = gtk.Label('')
        server_label.set_alignment (1.0, 0.5)
        server_label.set_markup('<b>Server</b>')
        data_table.attach (server_label, 0, 1, 1, 2,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        self.server_text = gtk.Label(self.ldap_server)
        self.server_text.set_alignment (1.0, 0.5)
        data_table.attach (self.server_text, 1, 2, 1, 2,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)

        configuration_id_label = gtk.Label('                     ')
        configuration_id_label.set_alignment (1.0, 0.5)
        configuration_id_label.set_markup('<b>Configuration</b>')
        data_table.attach (configuration_id_label, 0, 1, 2, 3,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        self.configuration_id_text = gtk.Label('         ')
        self.configuration_id_text.set_alignment (1.0, 0.5)
        data_table.attach (self.configuration_id_text, 1, 2, 2, 3,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)

        self.version_label = gtk.Label('')
        self.version_label.set_alignment (1.0, 0.5)
        self.version_label.set_markup('<b>Version</b>')
        data_table.attach (self.version_label, 0, 1, 3, 4,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        self.version_text = gtk.Label('            ')
        self.version_text.set_alignment (1.0, 0.5)
        data_table.attach (self.version_text, 1, 2, 3, 4,
                           gtk.FILL | gtk.EXPAND,
                           gtk.FILL | gtk.EXPAND)
        return data_table

    def create_window(self, debug=False):
        s = gtk.gdk.screen_get_default()
        height = s.get_height()
        width = s.get_width ()

        if debug:
            height = 600
            width = 800

        # create a psuedo-anaconda window
        if debug:
            window = gtk.Window()
        else:
            window = gtk.Window(gtk.WINDOW_POPUP)
        window.resize (width, height)

        window.move (0, 0)

        main_vbox = gtk.VBox (False, 0)
        window.add (main_vbox)

        # Pack the header
        pixbuf = gtk.gdk.pixbuf_new_from_file (get_pixmap_path ("anaconda_header.png"))
        if pixbuf.get_width() != width:
            pixbuf = pixbuf.scale_simple(width, pixbuf.get_height(), gtk.gdk.INTERP_BILINEAR)
        image = gtk.Image ()
        image.set_from_pixbuf (pixbuf)
        main_vbox.pack_start (image, False, False)

        # Pack the Main content area
        ## Help Text first
        table = gtk.Table (1, 3, gtk.TRUE)
        main_vbox.pack_start (table, True, True)
        swindow = gtk.ScrolledWindow ()
        swindow.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swindow.set_shadow_type (gtk.SHADOW_IN)
        swindow.set_border_width (4)
        table.attach (swindow, 0, 1, 0, 1,
                      gtk.FILL | gtk.EXPAND,
                      gtk.FILL | gtk.EXPAND)
        text_view = get_help_text ()
        swindow.add (text_view)

        # Progress area
        vbox = gtk.VBox (False, 10)
        vbox.set_border_width (10)
        frame = gtk.Frame ()
        frame.set_shadow_type(gtk.SHADOW_IN)
        image = gtk.Image ()
        pixbuf = gtk.gdk.pixbuf_new_from_file (get_pixmap_path ("progress_first.png"))
        image = gtk.Image ()
        image.set_from_pixbuf (pixbuf)
        frame.add (image)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        align.add (frame)
        vbox.pack_start (align, False, False)
        self.progress = gtk.ProgressBar ()
        vbox.pack_start (self.progress, True, False)
        self.label = gtk.Label('Contacting server...')
        self.label.set_alignment (0.0, 0.5)
        vbox.pack_start (self.label, True, False)
        table.attach (vbox, 1, 3, 0, 1,
                      gtk.FILL | gtk.EXPAND,
                      gtk.FILL | gtk.EXPAND)

        vbox.pack_start (self.create_data_table(), True, False)
                
        # Create the Button Area
        button_hbox = gtk.HBox (False, 24)
        button_hbox.set_border_width (5)
        def make_button (stock, text):
            button = gtk.Button()
            box = gtk.HBox(False, 0)
            image = gtk.Image()
            image.set_from_stock(stock, gtk.ICON_SIZE_BUTTON)
            box.pack_start(image, False, False)
            label_text = gtk.Label(text)
            label_text.set_property("use-underline", True)
            box.pack_start(label_text, True, True)
            button.add(box)
            button.set_sensitive (False)
            button.show_all()
            return button

        b = make_button ('gtk-help', "Hide _Help")
        button_hbox.pack_start (b, False, False)
        b = make_button ('gtk-new', "_Release Notes")
        button_hbox.pack_start (b, False, False)

        button_box = gtk.HButtonBox ()
        button_box.set_layout (gtk.BUTTONBOX_END)
        button_box.set_spacing (30)
        b = make_button ('gtk-go-back', "_Back")
        button_box.add (b)
        b = make_button ('gtk-go-forward', "_Next")
        button_box.add (b)
        button_hbox.pack_end (button_box, True, True)
        main_vbox.pack_end (button_hbox, False, False)

        main_vbox.show_all ()
        window.show_now ()

    def state_change_cb (self, r, state):
        state_change_mapping = {
            replicator.STATE_INIT : 'Initializing',
            replicator.STATE_SYNCING_ROOT : 'Copying over the system',
            replicator.STATE_SYNCING_BOOT : 'Copying over the kernel',
            replicator.STATE_POSTPROCESS : 'Setting up the bootloader',
            replicator.STATE_FINISHED : 'Copying complete',
            replicator.STATE_ERROR : 'Error'
            }
        try:
            label = state_change_mapping [state]
        except KeyError:
            label = ''
        print label
        self.label.set_text (label)

def run(ldap_server, root_dn, debug=False):
    gui = BootstrapGUI(ldap_server, root_dn, debug)
    gui.run()

if __name__ == '__main__':
    run ('aware-of-vacuity.boston.redhat.com', 'dc=sml-demo,dc=devel,dc=redhat,dc=com', True)


