#!/usr/bin/python
## Copyright (C) 2004 Red Hat, Inc.
 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
import sys
import os
import signal
import xml.dom.minidom
import gobject
import tempfile

import statelessgrub
import MACAddress
import LDAPUtils
import TransferMethod

from config import *

STATE_INIT = "initializing"
STATE_SYNCING_ROOT = "syncing-root"
STATE_SYNCING_BOOT = "syncing-boot"
STATE_POSTPROCESS = "postprocess"
STATE_FINISHED = "finished"
STATE_ERROR = "error"

def debug_print(msg):
    pass
    #print msg

def mount(device_name, mount_point, fs):
    command = "%s -t %s %s %s" % (normalize_path ('/bin/mount'), fs, device_name, mount_point)
    result = os.system (command)
    if result != 0:
        raise SystemError

def umount(mount_point):
    command = "%s %s" % (normalize_path ('/bin/umount'), mount_point)
    result = os.system (command)
    if result != 0:
        raise SystemError

class MountedDevice:
    def __init__(self,device_name,mount_point,fs):

        self.device_name = device_name
        self.mount_point = mount_point
        self.is_mounted = False

        debug_print ("Mounting %s at %s (type %s)"%(device_name, mount_point, fs))
        mount(device_name, mount_point, fs)
        self.is_mounted = True

    def __del__(self):
        if self.is_mounted:
            self.unmount()

    def unmount(self):
        debug_print ("Unmounting %s"%(self.mount_point))
        if self.is_mounted:
            umount(self.mount_point)
            self.is_mounted = False
            

#sys.path += ['/usr/lib/anaconda']
#import iutil
#import partedUtils
#import isys

import types, select

# Code taken from Anaconda: iutil.py
def getfd(filespec, readOnly = 0):
    if type(filespec) == types.IntType:
	return filespec
    if filespec == None:
	filespec = "/dev/null"

    flags = os.O_RDWR | os.O_CREAT
    if (readOnly):
	flags = os.O_RDONLY
    return os.open(filespec, flags)

# Code taken from Anaconda: iutil.py
def execWithRedirect(command, argv, stdin = 0, stdout = 1, stderr = 2,	
		     searchPath = 0, root = '/', newPgrp = 0,
		     ignoreTermSigs = 0):
    if not searchPath and not os.access (root + command, os.X_OK):
	raise RuntimeError, command + " can not be run"

    childpid = os.fork()
    if (not childpid):
        if (root and root != '/'): 
	    os.chroot (root)
	    os.chdir("/")

	if ignoreTermSigs:
	    signal.signal(signal.SIGTSTP, signal.SIG_IGN)
	    signal.signal(signal.SIGINT, signal.SIG_IGN)

        stdin = getfd(stdin)
        if stdout == stderr:
            stdout = getfd(stdout)
            stderr = stdout
        else:
            stdout = getfd(stdout)
            stderr = getfd(stderr)

	if stdin != 0:
	    os.dup2(stdin, 0)
	    os.close(stdin)
	if stdout != 1:
	    os.dup2(stdout, 1)
	    if stdout != stderr:
		os.close(stdout)
	if stderr != 2:
	    os.dup2(stderr, 2)
	    os.close(stderr)

        try:
            if (searchPath):
                os.execvp(command, argv)
            else:
                os.execv(command, argv)
        except OSError:
            # let the caller deal with the exit code of 1.
            pass

	os._exit(1)

    if newPgrp:
	os.setpgid(childpid, childpid)
	oldPgrp = os.tcgetpgrp(0)
	os.tcsetpgrp(0, childpid)

    status = -1
    try:
        (pid, status) = os.waitpid(childpid, 0)
    except OSError, (errno, msg):
        print __name__, "waitpid:", msg

    if newPgrp:
	os.tcsetpgrp(0, oldPgrp)

    return status

# Code taken from Anaconda: iutil.py
def execWithCapture(command, argv, searchPath = 0, root = '/', stdin = 0,
		    catchfd = 1, closefd = -1):

    if not searchPath and not os.access (root + command, os.X_OK):
	raise RuntimeError, command + " can not be run"

    (read, write) = os.pipe()

    childpid = os.fork()
    if (not childpid):
        if (root and root != '/'): os.chroot (root)
	os.dup2(write, catchfd)
	os.close(write)
	os.close(read)

	if closefd != -1:
	    os.close(closefd)

	if stdin:
	    os.dup2(stdin, 0)
	    os.close(stdin)

	if (searchPath):
	    os.execvp(command, argv)
	else:
	    os.execv(command, argv)

	os._exit(1)

    os.close(write)

    rc = ""
    s = "1"
    while (s):
	select.select([read], [], [])
	s = os.read(read, 1000)
	rc = rc + s

    os.close(read)

    try:
        os.waitpid(childpid, 0)
    except OSError, (errno, msg):
        print __name__, "waitpid:", msg

    return rc

def relabel_device(device, label):
    command = "%s %s %s" % (normalize_path ("/sbin/e2label"), device, label)
    debug_print ("relabelling %s to \"%s\""%(device, label))
    result = os.system (command)
    if result != 0:
        raise SystemError


def get_device_label(device):
    command = normalize_path ("/sbin/e2label")
    # FIXME: need to add some error checking here:
    foo = execWithCapture(command,
                          [command, device])
    # Strip off extra carriage returns (and other stuff?)
    return foo.strip()

def force_reboot():
    debug_print("Forcing a reboot")
    os.system("shutdown -r now")

# We expect the server image to contain a file /etc/sysconfig/stateless-snapshot
# The client adds a file /etc/sysconfig/snapshot-complete at the end of an rsync process
# and deletes it at the beginning; this signifies whether the snapshot is known to be good

SYSCONFIG_PATH = "/etc/sysconfig"
MARKER_FILE = "snapshot_complete"

ROLE_ROOT = "role-root"
ROLE_BOOT = "role-boot"

STATUS_ACTIVE = "status-active"
STATUS_RESERVE = "status-reserve"

def is_valid_role (role):
    return role==ROLE_ROOT or role==ROLE_BOOT

def is_valid_status (status):
    return status==STATUS_ACTIVE or status==STATUS_RESERVE


# Here is a family of exceptions to cover the various problems that can arise:
class ReplicatorException(Exception):
    "A base class for all exceptions raised by the replicator."
    def __init__(self, *args):
        Exception.__init__(self,*args)
        self.wrapped_exc = sys.exc_info()

class PartitionError(ReplicatorException):
    "Used to indicate problems determining which partition is which on the client."
    def __init__(self, *args):
        ReplicatorException.__init__(self,*args)

class LDAPError(ReplicatorException):
    "Used to indicate problems with the LDAP server."
    def __init__(self, *args):
        ReplicatorException.__init__(self,*args)

# A class representing the information in a stateless-snapshot file:
class SnapshotID:
    def __init__(self,file_name):
        f = open (file_name)
        lines = f.readlines()
        f.close()
        for line in lines:
            self.process_line (line)

    def process_line(self, line):
        # handle comments:
        if line[0] == '#':
            return

        assignment = line.strip().split('=')
        lhs = assignment[0]
        rhs = assignment[1]
        
        if (lhs=="STATELESS_CONFIGURATION"):
            self.config_name = rhs

        if (lhs=="STATELESS_SNAPSHOT"):
            self.snapshot_name = rhs

# A class representing the information stored within a client filesystem about a snapshot (either for a root or a boot partition):
class Snapshot:
    def __init__(self,snapshot_path):
        self.cfg = StatelessConfig()
        self.snapshot_path = snapshot_path
        self.snapshot_id = SnapshotID(snapshot_path + "/stateless-snapshot")
        self.is_complete = os.path.exists(snapshot_path + "/" + MARKER_FILE)

    # Are the config_name and server the same?
    def matches_provisioning(self, provisioning):
        return (self.snapshot_id.config_name == provisioning.config_name) and (self.snapshot_id.snapshot_name == provisioning.snapshot_name)

    def make_ldap_dir(self):
        return LDAPUtils.ReadOnlyDirectory(self.cfg)

def get_device_by_label(search_label):
    devices = ['/dev/hda1', '/dev/hda2', '/dev/hda3', '/dev/hda4', '/dev/hda5', '/dev/hda6'] # FIXME
    for dev in devices:
        label = get_device_label(dev)
        #print "Device \"%s\" has label \"%s\""%(dev, label)
        if label==search_label:
            return dev
    raise PartitionError

def on_rsync_update(rsync, pc, client_partition_role):
    client_partition_role.update(pc)

def on_rsync_finished(rsync, success, client_partition_role):
    debug_print("rsync finished")
    if (success):
        client_partition_role.write_marker_file()
    client_partition_role.finished(success)

def on_role_update(role, pc, replicator):
    replicator.update(pc)

def on_role_finished(role, success, replicator):
    debug_print("role finished")
    if not success:
        replicator.set_state(STATE_ERROR)
        replicator.finished(success)
        return
    if replicator.state==STATE_SYNCING_ROOT:
        replicator.root.disconnect(replicator.update_handler_id)
        replicator.root.disconnect(replicator.finished_handler_id)
        replicator.set_state(STATE_SYNCING_BOOT)
        replicator.boot.begin_update_client(replicator.provisioning)
        replicator.update_handler_id = replicator.boot.connect("update", on_role_update, replicator)
        replicator.finished_handler_id = replicator.boot.connect("finished", on_role_finished, replicator)
    else:
        assert replicator.state==STATE_SYNCING_BOOT
        replicator.boot.disconnect(replicator.update_handler_id)
        replicator.boot.disconnect(replicator.finished_handler_id)
        replicator.set_state(STATE_POSTPROCESS)
        replicator.do_postprocess()
        replicator.set_state(STATE_FINISHED)
        replicator.finished(success)

def dump_snapshot_status(title, device, snapshot):
    if (snapshot!=None):
        if (snapshot.is_complete):
            status_str = "complete"
        else:
            status_str = "incomplete"
        debug_print("%s on device \"%s\" ID:\"%s\" Version: %s (%s)" % \
              (title, device, snapshot.snapshot_id.config_name, snapshot.snapshot_id.snapshot_name, status_str))
    else:
        debug_print("%s not found"%(title))
        


class Replicator (gobject.GObject):
    __gsignals__ = {
        'update' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_FLOAT, )),
        'state-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, )),
        'finished' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN,))
        }

    class ClientPartitionRole (gobject.GObject):
        "A class for handling either the root or the boot partitions during client replication"

        __gsignals__ = {
            'update' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_FLOAT, )),
            'finished' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN,))
            }
            
        def __init__(self, role, tmpdir, replicator):
            gobject.GObject.__init__ (self)
            assert is_valid_role(role)
            self.role = role
            self.replicator = replicator

            # Create mountpoint:
            if self.role==ROLE_ROOT:
                self.dir_name_reserve = tmpdir + "/reserve_root"
            else:
                self.dir_name_reserve = tmpdir + "/reserve_boot"
            os.mkdir(self.dir_name_reserve)

            # Locate the active and reserve partitions for this role:
            self.locate_partitions()

            if not self.replicator.debug:
                # Mount the reserve partition in our temp directory:
                self.mounted_reserve = MountedDevice(self.reserve_device, self.dir_name_reserve, "ext3") #FIXME

                self.get_local_snapshot_info()
            
        def locate_partitions(self):
            # Locate the various partitions by role:
            try:
                self.active_device = self.get_device(STATUS_ACTIVE)
            except PartitionError:
                raise PartitionError, "Could not determine which %s is active"%self.get_name()

            try:
                self.reserve_device = self.get_device(STATUS_RESERVE)
            except PartitionError:
                raise PartitionError, "Could not determine which %s is in reserve"%self.get_name()

        def get_snapshot_path(self, status):
            assert is_valid_status(status)
            if self.role==ROLE_ROOT:
                if status==STATUS_ACTIVE:
                    return SYSCONFIG_PATH
                else:
                    return self.dir_name_reserve + SYSCONFIG_PATH
            else:
                if status==STATUS_ACTIVE:
                    return "/boot"
                else:
                    # The files are located in the root directory of a /boot partition:
                    return self.dir_name_reserve

        # Get the path on the server of a snapshot of a named system:
        def get_server_snapshot_path(self, name, version):
            snapshot_root_path = "/srv/stateless/snapshots/%s/%s-%s/"%(name, name, version)
            if self.role==ROLE_ROOT:
                return snapshot_root_path
            else:
                return snapshot_root_path + "boot/"

        def get_local_snapshot_info(self):
            # Determine version and status stored on the partitions:
            # Do the active partition:    
            try:
                self.active_snapshot = Snapshot(self.get_snapshot_path(STATUS_ACTIVE))
            except IOError:
                # It's OK for the active partition not to be fully set up, since we could be bootstrapping a new client machine:
                debug_print("Could not determine client snapshot information for active %s"%self.get_name())
                self.active_snapshot = None

            # Do the reserve partition:
            try:
                self.reserve_snapshot = Snapshot(self.dir_name_reserve + SYSCONFIG_PATH)
            except IOError:
                # It's OK for the reserve partition not to be fully set up:
                debug_print("Could not determine client snapshot information for reserve %s"%self.get_name())
                self.reserve_snapshot = None

        def get_name(self):
            if self.role==ROLE_ROOT:
                return "root partition"
            else:
                return "boot partition"

        def cleanup(self):
            self.mounted_reserve.unmount()
            os.rmdir(self.dir_name_reserve)

        def update (self, percent):
            self.emit ('update', percent)

        def finished (self, success):
            self.is_finished=True
            self.emit ('finished', success)

        def dump_status(self):
            dump_snapshot_status ("Active Snapshot ", self.get_device(STATUS_ACTIVE), self.active_snapshot)
            dump_snapshot_status ("Reserve Snapshot", self.get_device(STATUS_RESERVE), self.reserve_snapshot)                

        def get_device_label(self, status):
            assert is_valid_role(self.role)
            assert is_valid_status(status)

            if self.role==ROLE_ROOT:
                if status==STATUS_ACTIVE:
                    return "/"
                else:
                    return "RESERVE_ROOT"
            else:
                if status==STATUS_ACTIVE:
                    return "/boot"
                else:
                    return "RESERVE_BOOT"

        def get_device(self, status):
            if self.replicator.debug:
                return None
            else:
                device_label = self.get_device_label (status)
                return get_device_by_label (device_label)

        def is_active_up_to_date(self, provisioning):
            if (self.active_snapshot!=None):
                return (self.active_snapshot.matches_provisioning(provisioning) and (self.active_snapshot.is_complete))
            else:
                return False
        
        def is_reserve_up_to_date(self, provisioning):
            if (self.reserve_snapshot!=None):
                # OK: we have information on both the active and reserve partitions:
                if (self.reserve_snapshot.matches_provisioning(provisioning) and (self.reserve_snapshot.is_complete)):
                    return True
                else:
                    return False
            else:
                # We have a reserve partition, but the relevant files weren't found
                # We have an incomplete copy on the partition (or mislabelling has occurred)
                return False

        def begin_sync_version(self, provisioning, dest_device):
            debug_print("Getting %s to device %s"%(provisioning, dest_device))

            if self.role==ROLE_BOOT:
                self.marker_file_name = self.dir_name_reserve  + "/" + MARKER_FILE
            else:
                self.marker_file_name = self.dir_name_reserve + SYSCONFIG_PATH + "/" + MARKER_FILE

            # Remove marker file to signify an incomplete copy (in case of interruptions):
            try:
                os.remove(self.marker_file_name)
            except OSError:
                # Gracefully handle the case where file doesn't exist:
                pass

            # Get rsync dir from the LDAP directory
            try:
                (server_name, snapshots_path) = self.replicator.stateless_service.get_snapshot_location(provisioning.config_name,
                                                                                                        provisioning.snapshot_name,
                                                                                                        "rsync")
            except:
                raise LDAPError

            # Set up the sync method, and connect to signals:
            # Note that path should have a trailing slash
            rsync_url = "rsync://%s%s/%s/%s/" % (server_name,
                                                 snapshots_path,
                                                 provisioning.config_name,
                                                 provisioning.snapshot_name)
            if self.role==ROLE_BOOT:
                rsync_url = rsync_url + "boot/"
                exclude_options = ""
            else:
                # We want to exclude all the files in /boot, but we
                # need to include the directory as a mount point
                exclude_options = "--exclude=/boot/*"
                
            debug_print ("rsync url:\"%s\""%rsync_url)
            if (self.role==ROLE_ROOT):
                debug_print ("excluding /boot")

            if self.replicator.debug:
                self.rsync = TransferMethod.FakeMethod ()
            else:
                self.rsync = TransferMethod.RsyncMethod (rsync_url, self.dir_name_reserve, exclude_options)            
            self.rsync.connect("update", on_rsync_update, self)
            self.rsync.connect("transfer-finished", on_rsync_finished, self)
            self.rsync.start_transfer()

        # call this on a cronjob, or upon boot:
        def begin_update_client (self, provisioning):
            self.begin_sync_version(provisioning, self.reserve_device)

        def write_marker_file(self):
            # The image should now be good, so recreate the marker file:
            if not self.replicator.debug:
                marker_file = open(self.marker_file_name, "w")
                marker_file.write("Up-to-date") # FIXME: is there some meaningful content that this file should have?
                marker_file.close()


    def __init__ (self, ldap_dir, debug):
        gobject.GObject.__init__ (self)

        self.state = STATE_INIT
        self.ldap_dir = ldap_dir
        self.debug = debug
        self.stateless_service = LDAPUtils.StatelessService(ldap_dir)
        self.is_finished = False
        self.src_mount = None

        self.mac_address = MACAddress.get_eth_mac_address()
        
        # Create a single tempdir which will hold our various mountpoints
        self.temp_dir_name = tempfile.mkdtemp()
        self.root = Replicator.ClientPartitionRole(ROLE_ROOT, self.temp_dir_name, self)
        self.boot = Replicator.ClientPartitionRole(ROLE_BOOT, self.temp_dir_name, self)

        self.get_provisioning()

    def __del__(self):
        debug_print("Replicator.__del__")

    def cleanup(self):
        # Since __del__ doesn't seem to be being called let's make the cleanup explicit:
        self.root.cleanup()
        self.boot.cleanup()
        if self.src_mount!=None:
            self.src_mount.unmount()

    def update (self, percent):
        self.emit ('update', percent)

    def finished (self, success):
        self.is_finished=True
        self.emit ('finished', success)

    def cancel_transfer (self):
        pass

    def do_postprocess(self):
        raise NotImplementedError


    def get_provisioning(self):
        self.provisioning = self.stateless_service.get_client_provisioning(self.mac_address)
        debug_print ("Provisioning data from server:%s"%self.provisioning)

    def is_active_up_to_date(self):
        assert self.provisioning
        return self.root.is_active_up_to_date(self.provisioning) and self.boot.is_active_up_to_date(self.provisioning)

    def is_reserve_up_to_date(self):
        assert self.provisioning
        return self.root.is_reserve_up_to_date(self.provisioning) and self.boot.is_reserve_up_to_date(self.provisioning)

    def begin_update_client(self):
        self.set_state(STATE_SYNCING_ROOT)
        self.update_handler_id = self.root.connect("update", on_role_update, self)
        self.finished_handler_id = self.root.connect("finished", on_role_finished, self)
        self.root.begin_update_client(self.provisioning)

    def set_state(self, state):
        self.state = state
        self.emit("state-change", state)

    def do_device_relabelling(self):
        # actives become reserves, and vice versa
        relabel_device (self.root.active_device, "RESERVE_ROOT")
        relabel_device (self.root.reserve_device, "/")
        relabel_device (self.boot.active_device, "RESERVE_BOOT")
        relabel_device (self.boot.reserve_device, "/boot")

    def do_relabel_and_grub(self):
        if not self.debug:
            self.do_device_relabelling()

            # boot needs to be mounted inside root for statelessGrub
            self.boot.mounted_reserve.unmount()
            mount(self.boot.reserve_device,
                  self.root.dir_name_reserve + "/boot",
                  "ext3")
            
            s = statelessgrub.statelessGrub (self.boot.reserve_device,
                                             self.root.dir_name_reserve)
            s.reinstall()
            
            umount(self.root.dir_name_reserve + "/boot")

gobject.type_register (Replicator.ClientPartitionRole)
gobject.type_register (Replicator)

class BootstrapReplicator(Replicator):
    "A Replicator for doing the initial sync of a fresh client install."
    def __init__(self, cfg, debug):
        ldap_dir = LDAPUtils.ReadOnlyDirectory(cfg)
        Replicator.__init__ (self, ldap_dir, debug)

    def do_postprocess(self):
        self.do_relabel_and_grub()


class LiveReplicator(Replicator):
    "A Replicator for doing (lazy) updates of a running client."
    def __init__(self):
        snapshot = Snapshot(SYSCONFIG_PATH)
        ldap_dir = snapshot.make_ldap_dir()
        Replicator.__init__ (self, ldap_dir, False)

class LiveReplicatorBackgroundUpdate(LiveReplicator):
    def do_postprocess(self):
        # We have obtained a more up-to-date image into the reserve partitions, relabel, regrub, but don't reboot
        self.do_relabel_and_grub()

class LiveReplicatorUpdateOnBoot(LiveReplicator):
    def do_postprocess(self):
        # We are booting, and a more up-to-date image has been updated.  Relabel partitions, run grub...
        self.do_relabel_and_grub()

        # And now reboot to use the new image:
        force_reboot()

class LiveReplicatorStatus(LiveReplicator):
    def do_postprocess(self):
        # This should never be reached; get pychecker to shut up:
        assert False

# We can be invoked in four ways:
# (i) by jrb's post-install tool that does an initial rsync, with a GTK frontend
# (ii) on boot, with a textual frontend
# (iii) on a cronjob, with no frontend
# (iv) manually, probably with a textual frontend, though a GTK frontend could be possible as well (for use in case an update isn't working for some reason)

def do_quit(replicator, success, text_ui):
    text_ui.loop.quit()

class ReplicatorTextUI:
    def __init__(self, repl):
        self.is_finished = False
        self.repl = repl

    def run(self):
        self.repl.connect('finished', do_quit, self)
        self.repl.begin_update_client()
        if not self.repl.is_finished:
            self.loop = gobject.MainLoop()
            self.loop.run()


def update_client_on_boot():
    # On boot we want to minimise activity, so that the user can get into their working environment ASAP:
    # FIXME: do we want an idea of non-critical updates?
    repl = LiveReplicatorUpdateOnBoot()
    if repl.is_active_up_to_date():
        # Active is up-to-date, don't bother doing anything more for now:
        pass
    else:        
        if repl.is_reserve_up_to_date():
            debug_print("Reserve snapshot is up-to-date")
            # OK - we have a more up-to-date image in reserve: relabel, regrub, and reboot:
            repl.do_relabel_and_grub()
            force_reboot()
        else:
            text_ui = ReplicatorTextUI(repl)
            text_ui.run()            
    repl.cleanup()    

def update_client_on_cronjob_or_manual():
    # On a cronjob or manual update we always want to try updating the reserve:
    repl = LiveReplicatorBackgroundUpdate()
    if repl.is_reserve_up_to_date():
        debug_print("Reserve snapshot is up-to-date")
    else:
        # We should sync the latest version onto the reserve partition (even if it's the same
        # version as on the active partition - having both partitions up-to-date is a good thing)
        text_ui = ReplicatorTextUI(repl)
        text_ui.run()
    repl.cleanup()    
        
def get_status():
    "Routine for getting status of partitions"
    repl = LiveReplicatorStatus()
    debug_print("Root Partitions")
    repl.root.dump_status()
    debug_print("Boot Partitions")
    repl.boot.dump_status()
    repl.cleanup()
    
    

#update_client_on_cronjob()

if __name__ == '__main__':
    if len(sys.argv)>1:
        if sys.argv[1]=="status":
            get_status()
        elif sys.argv[1]=="boot":
            update_client_on_boot()
        elif sys.argv[1]=="update":
            update_client_on_cronjob_or_manual()
        else:
            print "Usage: update-client.py [status|boot|update]"
    else:
        get_status()

